@@ -1,9 +1,9 @@
 /*
- *  $Id: dlg_keys.h,v 1.21 2007/02/21 22:40:29 tom Exp $
+ *  $Id: dlg_keys.h,v 1.26 2011/06/21 22:09:22 tom Exp $
  *
  *  dlg_keys.h -- runtime binding support for dialog
  *
- *  Copyright 2005	Thomas E. Dickey
+ *  Copyright 2005-2010,2011 Thomas E.  Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -28,13 +28,17 @@
 
 #ifdef USE_WIDE_CURSES
 #include <wctype.h>
-#define dlg_toupper(ch) towupper(ch)
-#define dlg_isupper(ch) iswupper(ch)
+#define dlg_toupper(ch) towupper((wint_t)ch)
+#define dlg_isupper(ch) iswupper((wint_t)ch)
 #else
 #define dlg_toupper(ch) toupper(ch)
 #define dlg_isupper(ch) (isalpha(ch) && isupper(ch))
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct {
     int is_function_key;
     int	curses_key;
@@ -85,12 +89,18 @@ typedef enum {
     DLGK_BEGIN,
     DLGK_FINAL,
     DLGK_SELECT,
+    DLGK_HELPFILE,
     DLGK_TRACE
 } DLG_KEYS_ENUM;
 
 #define is_DLGK_MOUSE(code)	((code) >= M_EVENT)
 #define DLGK_MOUSE(code)	((code) + M_EVENT)
 
+#define HELPKEY_BINDINGS \
+	DLG_KEYS_DATA( DLGK_HELPFILE,	   CHR_HELP ), \
+	DLG_KEYS_DATA( DLGK_HELPFILE,	   KEY_F(1) ), \
+	DLG_KEYS_DATA( DLGK_HELPFILE,	   KEY_HELP )
+
 #define ENTERKEY_BINDINGS \
 	DLG_KEYS_DATA( DLGK_ENTER,	   '\n' ), \
 	DLG_KEYS_DATA( DLGK_ENTER,	   '\r' ), \
@@ -108,6 +118,24 @@ typedef enum {
 	DLG_KEYS_DATA( DLGK_GRID_LEFT,	   KEY_LEFT ), \
 	DLG_KEYS_DATA( DLGK_GRID_RIGHT,	   KEY_RIGHT )
 
+#define SCROLLKEY_BINDINGS \
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	'J' ), \
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	'j' ), \
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	KEY_DOWN ), \
+	DLG_KEYS_DATA( DLGK_GRID_UP,	'K' ), \
+	DLG_KEYS_DATA( DLGK_GRID_UP,	'k' ), \
+	DLG_KEYS_DATA( DLGK_GRID_UP,	KEY_UP ), \
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST,	'g' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_FIRST,	KEY_HOME ), \
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,	'G' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_LAST,	KEY_END ), \
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	'F' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	'f' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,	KEY_NPAGE ), \
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	'B' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	'b' ), \
+	DLG_KEYS_DATA( DLGK_PAGE_PREV,	KEY_PPAGE )
+
 extern int dlg_lookup_key(WINDOW * /*win*/, int /*curses_key*/, int * /*dialog_key*/);
 extern int dlg_result_key(int /*dialog_key*/, int /*fkey*/, int * /*resultp*/);
 extern void dlg_register_buttons(WINDOW * /*win*/, const char * /*name*/, const char ** /*buttons*/);
@@ -119,4 +147,8 @@ extern int dlg_parse_bindkey(char * /*params*/);
 extern void dlg_dump_keys(FILE * /*fp*/);
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* DLG_KEYS_H_included */
